@@ -25,6 +25,10 @@ namespace ept {
 namespace core {
 
 struct AptException : wibble::exception::Generic {
+    std::string desc() const throw () {
+        return "APT Error";
+    }
+
     AptException( const std::string &ctx ) : Generic( ctx ) {
         while ( !_error->empty() ) {
             std::string err;
@@ -47,7 +51,8 @@ struct PackageState {
         WillBreak = 1 << 7,
         ReInstall = 1 << 8,
         Purge = 1 << 9,
-        Hold = 1 << 10
+        Hold = 1 << 10,
+        Valid = 1 << 11
     };
 
     typedef unsigned state;
@@ -70,6 +75,11 @@ struct PackageState {
     
     PackageState() : m_state( 0 ) {}
 
+    // FIXME this probably needs to be used consistently in core and out of core
+    bool isValid() const { return m_state & Valid; }
+    // FIXME compatibility API for non-core apt
+    bool isInstalled() const { return installed(); }
+
     bool install() const { return m_state & Install; }
     // reinstall() implies install()
     bool reinstall() const { return m_state & ReInstall; }
@@ -96,6 +106,8 @@ protected:
     unsigned m_state;
 };
 
+time_t aptTimestamp();
+
 // wrap the apt's database
 struct AptDatabase {
     pkgCache &cache() {
@@ -120,6 +132,10 @@ struct AptDatabase {
     bool m_tryWriteable;
     bool m_writeable;
 
+    time_t timestamp() {
+        return aptTimestamp();
+    }
+
     AptDatabase() {
         m_cache = 0;
         m_state = 0;
@@ -203,6 +219,11 @@ struct AptDatabase {
         m_cache = 0;
     }
 
+    pkgCache::VerIterator anyVersion( pkgCache::PkgIterator pi ) {
+	if ( pi.end() ) return pkgCache::VerIterator();
+        return pi.VersionList();
+    }
+
     Token candidateVersion( Token p ) {
 	pkgCache::PkgIterator pi = cache().FindPkg( p.package() );
 	if ( pi.end() ) return Token();
@@ -258,9 +279,15 @@ struct AptDatabase {
         return vfi;
     }
 
+    PackageState invalidState() {
+        return PackageState( PackageState::NowBroken | PackageState::Keep );
+    }
+
     PackageState packageState( pkgCache::PkgIterator P )
     {
         PackageState s = 0;
+        if ( P.end() )
+            return invalidState();
         if ( ! ( P->CurrentState == pkgCache::State::ConfigFiles
                  || P->CurrentState == pkgCache::State::NotInstalled ) )
             s |= PackageState::Installed;
@@ -289,7 +316,10 @@ struct AptDatabase {
     }
 
     PackageState packageState( Token t ) {
-        return packageState( lookupPackage( t ) );
+        t = validate( t );
+        if ( t.valid() )
+            return packageState( lookupPackage( t ) );
+        return invalidState();
     }
 
     Token validate( Token t ) {
@@ -321,6 +351,7 @@ struct AptInternalList {
     Internal &head() { return m_head; }
     bool empty() const { return m_head.end(); }
     AptInternalList( Internal head ) : m_head( head ) {}
+    AptInternalList() {}
 };
 
 namespace version {
